/*
    CozyCL, a comfortable OpenCL library
    Copyright (C) 2013  Tamas Bolner
	For more information, visit: http://blog.bolner.hu/2013/02/19/cozycl
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once
#include <string>
#include <vector>
#include <CL/opencl.h>
#include "exception.h"

typedef unsigned __int64 uInt64;
typedef unsigned __int32 uInt32;

using namespace std;

class CozyCL {
private:
	uInt32 cz_memoryObject_num;
	cl_platform_id cz_platform_id;
	cl_device_id cz_device_id;
	cl_context cz_context;
	cl_command_queue cz_queue;
	cl_program cz_program;
	cl_mem *cz_buffers;
	cl_kernel *cz_kernels;
	cl_event *cz_events;

	cl_program build_program(cl_context context, cl_device_id device_id, char *file_path);
	void cleanup();

	//Static members
	static void throwException(char *prefix, cl_int errorCode);

protected:

public:
	CozyCL(uInt32 platform_id, uInt32 device_id, char *programFilePath, char *kernel_func, uInt32 memoryObject_num, uInt64 memoryObject_memSize);
	~CozyCL();
	void writeMemory(uInt32 memoryObject_id, void *host_addr, uInt64 size, uInt64 device_addr);
	void readMemory(uInt32 memoryObject_id, void *host_addr, uInt64 size, uInt64 device_addr);
	void startKernel(uInt32 memoryObject_id, uInt32 global_size, uInt32 local_size);
	void waitForAll();

	//Static members
	static vector<string> getPlatforms();
	static vector<string> getDevices(uInt32 platform_id);
};
