/*
    CozyCL, a comfortable OpenCL library
    Copyright (C) 2013  Tamas Bolner
	For more information, visit: http://blog.bolner.hu/2013/02/19/cozycl
	
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include "CozyCL.hpp"

int main()
{
	vector<string> platforms = CozyCL::getPlatforms();
	vector<string> devices;
	CozyCL *cozyCl = NULL;
	
	/*
		Prepare test data
	*/
	char *test_data = new char[11];
	for (uInt32 i = 0; i < 10; i++) test_data[i] = 65;
	test_data[10] = 0;

	try {
		/*
			Print out list of Platforms and Devices
		*/
		std::cout << std::endl;
		for (int i = 0; i < platforms.size(); i++) {
			std::cout << "Platform " << i << ": " << platforms[i] << std::endl;

			devices = CozyCL::getDevices(0);
		
			for (int i = 0; i < devices.size(); i++) {
				std::cout << " - Device " << i << ": " << devices[i] << std::endl;
			}

			std::cout << std::endl;
		}

		std::cout << "Selected: Platform 0, Device 0" << std::endl << std::endl;

		/*
			Print input data
		*/
		std::cout << "Input:  " << test_data << std::endl;

		/*
			Run Program on Graphics Card
		*/
		cozyCl = new CozyCL(0, 0, "test_program.cl", "test_function", 1, 10);

		cozyCl->writeMemory(0, test_data, 10, 0);
		cozyCl->startKernel(0, 10, 5);

		cozyCl->waitForAll();

		/*
			Print result
		*/
		cozyCl->readMemory(0, test_data, 10, 0);
		std::cout << "Output: " << test_data << std::endl;

	} catch (FException *ex) {
		ex->Print();
	}

	/*
		The destructor of the CozyCL class will deallocate
		every OpenCL object automatically.
	*/
	delete cozyCl;
	delete test_data;

	return 0;
}
